//
//  QC3DPlugin.m
//  QCPlugin
//
//  Created by Pierre Chatelier on 29/05/08.
//  Copyright (c) 2008 Pierre Chatelier. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "QC3DPlugin.h"

#define	kQCPlugIn_Name				@"QC3DPlugin"
#define	kQCPlugIn_Description		@"QC3DPlugin description"

@implementation QC3DPlugin

@dynamic inputDeltaX;
@dynamic inputDeltaY;
@dynamic inputModelFilePath;
 
+(NSDictionary*) attributes
{
	return [NSDictionary dictionaryWithObjectsAndKeys:
           kQCPlugIn_Name, QCPlugInAttributeNameKey,
           kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}
//end attributes

+(NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
  NSDictionary* result = nil;
  if([key isEqualToString:@"inputDeltaX"])
    result = [NSDictionary dictionaryWithObjectsAndKeys:@"delta X", QCPortAttributeNameKey, nil];
  else if([key isEqualToString:@"inputDeltaY"])
    result = [NSDictionary dictionaryWithObjectsAndKeys:@"delta Y", QCPortAttributeNameKey, nil];
  else if([key isEqualToString:@"inputModelFilePath"])
    result = [NSDictionary dictionaryWithObjectsAndKeys:@"input Model filePath", QCPortAttributeNameKey,
                                                        @"", QCPortAttributeDefaultValueKey, nil];
  return result;
}
//end attributesForPropertyPortWithKey:

+(QCPlugInExecutionMode) executionMode
{
  //kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, kQCPlugInExecutionModeConsumer
  return kQCPlugInExecutionModeConsumer;
}
//end executionMode

+(QCPlugInTimeMode) timeMode
{
  //kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle, kQCPlugInTimeModeTimeBase
	return kQCPlugInTimeModeNone;
}
//end timeMode

-(id) init
{
	if(![super init])
    return nil;
	return self;
}
//end init

-(void) dealloc
{
  if (mesh)
    meshRelease(&mesh);
	[super dealloc];
}
//end dealloc

-(void) finalize
{
	[super finalize];
}
//end finalize

@end

@implementation QC3DPlugin (Execution)

-(BOOL) startExecution:(id<QCPlugInContext>)context
{
	return YES;
}
//end startExecution:

-(void) enableExecution:(id<QCPlugInContext>)context
{
}
//end enableExecution:

-(BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
  CGLContextObj cgl_ctx = [context CGLContextObj];
  glClearColor(0.0, 0.0, 0.0, 0.0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_BLEND);

  glMatrixMode(GL_PROJECTION);
  float fovyInDeg = 90;
  float f = cos(fovyInDeg/2*3.1415927/180)/sin(fovyInDeg/2*3.1415927/180);
  float aspect = [context bounds].size.width/[context bounds].size.height;
  float zNear = .1f;
  float zFar  = 100.f;
  GLfloat perspective[16] = {
    f/aspect, 0, 0, 0,
    0, f, 0, 0,
    0, 0, (zFar+zNear)/(zNear-zFar), -1,
    0, 0, (2*zFar*zNear)/(zNear-zFar), 0};
  glLoadMatrixf(&perspective[0]);

  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();
  glTranslatef(0.f, 0.f, -5.f);
  rotationAngleXDeg += 10*self.inputDeltaY;
  rotationAngleYDeg += 10*self.inputDeltaX;
  glRotatef(rotationAngleXDeg, 1.f, 0.f, 0.f);
  glRotatef(rotationAngleYDeg, 0.f, 1.f, 0.f);
  
  if (![cachedInputModelFilePath isEqualToString:self.inputModelFilePath])
  {
    [cachedInputModelFilePath release];
    cachedInputModelFilePath = [self.inputModelFilePath copy];
    meshRelease(&mesh);
    if (cachedInputModelFilePath)
      mesh = meshCreateFromFile([cachedInputModelFilePath UTF8String]);
  }
  if (mesh)
    meshDraw(cgl_ctx, mesh);
  glPopMatrix();
/*
  glScalef(2.f, 2.f, 1.f);
  glBegin(GL_QUADS);
    glColor3f(1.f, 0.f, 0.f);
    glVertex3f(-1.f, -1.f, 0.f);
    glColor3f(0.f, 1.f, 0.f);
    glVertex3f(-1.f,  1.f, 0.f);
    glColor3f(0.f, 0.f, 1.f);
    glVertex3f( 1.f,  1.f, 0.f);
    glColor3f(1.f, 1.f, 1.f);
    glVertex3f( 1.f, -1.f, 0.f);
  glEnd();
  glPopMatrix();*/
  
	return YES;
}
//end execute:atTime:withArguments:

-(void) disableExecution:(id<QCPlugInContext>)context
{
}
//end disableExecution:

-(void) stopExecution:(id<QCPlugInContext>)context
{
}
//end stopExecution:

@end
